# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test components."""

from invenio_access.permissions import system_identity

from invenio_communities.proxies import current_identities_cache
from invenio_communities.utils import identity_cache_key


def test_accept_invite_cache_clear(
    requests_service, invite_request_id, invite_user, db, search_clear
):
    """Test that the community member cached entries are cleared."""
    current_identities_cache.flush()
    cache_key = identity_cache_key(invite_user.identity)

    invite_user.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 0

    # cached entry should be cleared on accept_invite
    requests_service.execute_action(invite_user.identity, invite_request_id, "accept")
    community_roles = current_identities_cache.get(cache_key)
    assert community_roles is None
    invite_user.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 1


def test_member_delete_cache_clear(
    member_service, community, new_user, db, search_clear
):
    """Test that the community member cached entries are cleared."""
    current_identities_cache.flush()
    cache_key = identity_cache_key(new_user.identity)

    new_user.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 0

    # cached entry should be cleared on add
    add_data = {
        "members": [{"type": "user", "id": str(new_user.id)}],
        "role": "reader",
    }
    member_service.add(system_identity, community._record.id, add_data)
    community_roles = current_identities_cache.get(cache_key)
    assert community_roles is None
    new_user.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 1

    # cached entry should be cleared on delete
    delete_data = {
        "members": [{"type": "user", "id": str(new_user.id)}],
    }
    member_service.delete(system_identity, community._record.id, delete_data)
    community_roles = current_identities_cache.get(cache_key)
    assert community_roles is None
    new_user.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 0


def test_member_add_cache_clear(member_service, community, new_user, db, search_clear):
    """Test that the community member cached entries are cleared."""
    current_identities_cache.flush()
    cache_key = identity_cache_key(new_user.identity)

    new_user.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 0

    # cached entry should be cleared on add
    add_data = {
        "members": [{"type": "user", "id": str(new_user.id)}],
        "role": "reader",
    }
    member_service.add(system_identity, community._record.id, add_data)
    community_roles = current_identities_cache.get(cache_key)
    assert community_roles is None
    new_user.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 1


def test_member_update_cache_clear(
    member_service, community, new_user, db, search_clear
):
    """Test that the community member cached entries are cleared."""
    current_identities_cache.flush()
    cache_key = identity_cache_key(new_user.identity)

    new_user.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 0

    # cached entry should be cleared on add
    add_data = {
        "members": [{"type": "user", "id": str(new_user.id)}],
        "role": "reader",
    }
    member_service.add(system_identity, community._record.id, add_data)
    community_roles = current_identities_cache.get(cache_key)
    assert community_roles is None
    new_user.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 1

    update_data = {
        "members": [{"type": "user", "id": str(new_user.id)}],
        "role": "reader",
    }
    member_service.update(system_identity, community._record.id, update_data)
    community_roles = current_identities_cache.get(cache_key)
    assert community_roles is None
    new_user.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 1


def test_group_actions_cache_clear(
    member_service, restricted_community, admin, db, search_clear
):
    """Test that the community member cached entries are cleared on group actions."""
    current_identities_cache.flush()
    cache_key = identity_cache_key(admin.identity)

    admin.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 0

    # cached entry should be cleared for group members on add
    data = {
        "members": [{"type": "group", "id": "admin-access"}],
        "role": "reader",
    }

    # cached entry should be cleared for group members
    member_service.add(system_identity, restricted_community._record.id, data)
    community_roles = current_identities_cache.get(cache_key)
    assert community_roles is None

    admin.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 1

    # cached entry should be cleared for group members on update
    update_data = {
        "members": [{"type": "group", "id": "admin-access"}],
        "role": "manager",
    }
    member_service.update(system_identity, restricted_community._record.id, update_data)
    community_roles = current_identities_cache.get(cache_key)
    assert community_roles is None

    admin.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 1

    # cached entry should be cleared for group members on delete
    delete_data = {
        "members": [{"type": "group", "id": "admin-access"}],
    }
    member_service.delete(system_identity, restricted_community._record.id, delete_data)
    community_roles = current_identities_cache.get(cache_key)
    assert community_roles is None
    admin.refresh()
    community_roles = current_identities_cache.get(cache_key)
    assert len(community_roles) == 0



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
# Copyright (C) 2021 Northwestern University.
# Copyright (C) 2022 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Communities Resource API."""

from flask import g
from flask_resources import (
    from_conf,
    request_parser,
    resource_requestctx,
    response_handler,
    route,
)
from invenio_records_resources.resources.files.resource import request_stream
from invenio_records_resources.resources.records.resource import (
    RecordResource,
    request_data,
    request_extra_args,
    request_headers,
    request_search_args,
    request_view_args,
)
from invenio_records_resources.resources.records.utils import search_preference

from invenio_communities.proxies import current_communities

request_community_requests_search_args = request_parser(
    from_conf("request_community_requests_search_args"), location="args"
)


class CommunityResource(RecordResource):
    """Communities resource."""

    def create_url_rules(self):
        """Create the URL rules for the record resource."""
        routes = self.config.routes
        return [
            route("GET", routes["list"], self.search),
            route("POST", routes["list"], self.create),
            route("GET", routes["item"], self.read),
            route("PUT", routes["item"], self.update),
            route("DELETE", routes["item"], self.delete),
            route("GET", routes["user-communities"], self.search_user_communities),
            route("POST", routes["rename"], self.rename),
            route("GET", routes["logo"], self.read_logo),
            route("PUT", routes["logo"], self.update_logo),
            route("DELETE", routes["logo"], self.delete_logo),
            route("GET", routes["featured-search"], self.featured_communities_search),
            route("GET", routes["featured-list"], self.featured_list),
            route("POST", routes["featured-list"], self.featured_create),
            route("PUT", routes["featured-item"], self.featured_update),
            route("DELETE", routes["featured-item"], self.featured_delete),
            route("GET", routes["community-requests"], self.search_community_requests),
            route("POST", routes["restore-community"], self.restore_community),
        ]

    @request_search_args
    @response_handler(many=True)
    def search_user_communities(self):
        """Perform a search over the user's communities.

        GET /user/communities
        """
        hits = self.service.search_user_communities(
            identity=g.identity,
            params=resource_requestctx.args,
            search_preference=search_preference(),
        )
        return hits.to_dict(), 200

    @request_extra_args
    @request_view_args
    @request_community_requests_search_args
    @response_handler(many=True)
    def search_community_requests(self):
        """Perform a search over the community's requests.

        GET /communities/<pid_value>/requests
        """
        hits = self.service.search_community_requests(
            identity=g.identity,
            community_id=resource_requestctx.view_args["pid_value"],
            params=resource_requestctx.args,
            search_preference=search_preference(),
            expand=resource_requestctx.args.get("expand", False),
        )
        return hits.to_dict(), 200

    @request_headers
    @request_view_args
    @request_data
    @response_handler()
    def rename(self):
        """Rename a community."""
        item = self.service.rename(
            g.identity,
            resource_requestctx.view_args["pid_value"],
            resource_requestctx.data,
            revision_id=resource_requestctx.headers.get("if_match"),
        )
        return item.to_dict(), 200

    @request_view_args
    def read_logo(self):
        """Read logo's content."""
        community_pid = resource_requestctx.view_args["pid_value"]
        item = self.service.read_logo(
            g.identity,
            community_pid,
        )
        community = current_communities.service.read(
            id_=community_pid, identity=g.identity
        ).to_dict()

        is_restricted = community["access"]["visibility"] == "restricted"

        return item.send_file(restricted=is_restricted)

    @request_view_args
    @request_stream
    @response_handler()
    def update_logo(self):
        """Upload logo content."""
        item = self.service.update_logo(
            g.identity,
            resource_requestctx.view_args["pid_value"],
            resource_requestctx.data["request_stream"],
            content_length=resource_requestctx.data["request_content_length"],
        )
        return item.to_dict(), 200

    #
    # Deletion workflows
    #
    @request_headers
    @request_view_args
    @request_data
    def delete(self):
        """Read the related review request."""
        self.service.delete_community(
            g.identity,
            resource_requestctx.view_args["pid_value"],
            resource_requestctx.data,
            revision_id=resource_requestctx.headers.get("if_match"),
        )

        return "", 204

    @request_headers
    @request_view_args
    @request_data
    def restore_community(self):
        """Read the related review request."""
        item = self.service.restore_community(
            g.identity,
            resource_requestctx.view_args["pid_value"],
            resource_requestctx.data,
        )

        return item.to_dict(), 200

    @request_view_args
    def delete_logo(self):
        """Delete logo."""
        self.service.delete_logo(
            g.identity,
            resource_requestctx.view_args["pid_value"],
        )
        return "", 204

    @request_search_args
    @response_handler(many=True)
    def featured_communities_search(self):
        """Features communities search."""
        hits = self.service.featured_search(
            identity=g.identity,
            params=resource_requestctx.args,
            search_preference=search_preference(),
        )
        return hits.to_dict(), 200

    @request_headers
    @request_view_args
    @response_handler()
    def featured_list(self):
        """List featured entries for a community."""
        items = self.service.featured_list(
            g.identity,
            resource_requestctx.view_args["pid_value"],
        )
        return items.to_dict(), 200

    @request_headers
    @request_view_args
    @request_data
    @response_handler()
    def featured_create(self):
        """Create a featured community entry."""
        item = self.service.featured_create(
            g.identity,
            resource_requestctx.view_args["pid_value"],
            resource_requestctx.data,
        )
        return item.to_dict(), 201

    @request_headers
    @request_view_args
    @request_data
    @response_handler()
    def featured_update(self):
        """Update a featured community entry."""
        item = self.service.featured_update(
            g.identity,
            resource_requestctx.view_args["pid_value"],
            resource_requestctx.data,
            featured_id=resource_requestctx.view_args["featured_id"],
        )
        return item.to_dict(), 200

    @request_view_args
    def featured_delete(self):
        """Delete a featured community entry."""
        self.service.featured_delete(
            g.identity,
            resource_requestctx.view_args["pid_value"],
            featured_id=resource_requestctx.view_args["featured_id"],
        )
        return "", 204



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test community UI views."""
import pytest
from flask import Blueprint

from invenio_communities.views.communities import render_community_theme_template


@pytest.fixture()
def blueprint():
    """Blueprint for loading templates."""
    # This blueprint picks up the tests/templates/ folder, so that we can
    # render templates from inside this folder.
    return Blueprint("tests", __name__, template_folder="templates")


@pytest.fixture()
def theme_app(instance_path, blueprint, app):
    """Application with template theming."""
    app.config["APP_THEME"] = ["semantic-ui"]
    app.register_blueprint(blueprint)

    yield app


def test_template_loader(theme_app):
    """Test template loader."""

    @theme_app.route("/community_theme")
    def community_theme():
        return render_community_theme_template(
            "invenio_test/frontpage.html", theme={"brand": "horizon", "enabled": True}
        )

    @theme_app.route("/")
    def index():
        return render_community_theme_template("invenio_test/frontpage.html")

    c = theme_app.test_client()
    rv = c.get("/community_theme")
    assert b"TEST HORIZON FRONTPAGE" in rv.data
    rv = c.get("/")
    assert b"DEFAULT COMMUNITY FRONTPAGE" in rv.data



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
# Copyright (C) 2022 Northwestern University.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test community service."""

import time
import uuid
from copy import deepcopy
from datetime import datetime, timedelta

import arrow
import pytest
from invenio_access.permissions import system_identity
from invenio_cache import current_cache
from invenio_pidstore.errors import PIDDoesNotExistError
from invenio_records_resources.services.errors import PermissionDeniedError
from marshmallow import ValidationError

from invenio_communities.communities.records.systemfields.deletion_status import (
    CommunityDeletionStatusEnum,
)
from invenio_communities.communities.services.service import get_cached_community_slug
from invenio_communities.errors import (
    CommunityFeaturedEntryDoesNotExistError,
    DeletionStatusError,
)
from invenio_communities.fixtures.tasks import reindex_featured_entries


@pytest.fixture()
def comm(community_service, minimal_community, location):
    """Create minimal public community."""
    c = deepcopy(minimal_community)
    c["slug"] = "{slug}".format(
        slug=str(datetime.utcnow().timestamp()).replace(".", "-")
    )
    return community_service.create(data=c, identity=system_identity)


@pytest.fixture()
def comm_restricted(community_service, minimal_community, location):
    """Create minimal restricted community."""
    c = deepcopy(minimal_community)
    c["slug"] = "{slug}".format(
        slug=str(datetime.utcnow().timestamp()).replace(".", "-")
    )
    c["access"]["visibility"] = "restricted"
    return community_service.create(data=c, identity=system_identity)


def test_search_featured(community_service, comm, db, search_clear):
    """Test that featured entries are indexed and returned correctly."""
    data = {
        "start_date": datetime.utcnow().isoformat(),
    }
    future_data = {
        "start_date": (datetime.utcnow() + timedelta(days=1)).isoformat(),
    }

    # no featured entries -> no featured communities
    featured_comms = community_service.featured_search(
        identity=system_identity
    ).to_dict()
    assert len(featured_comms["hits"]["hits"]) == featured_comms["hits"]["total"] == 0

    # added one featured entry in the past. community should now be returned
    community_service.featured_create(
        identity=system_identity, community_id=comm.data["id"], data=data
    ).to_dict()
    featured_comms = community_service.featured_search(
        identity=system_identity
    ).to_dict()
    hits = featured_comms["hits"]["hits"]
    assert len(hits) == featured_comms["hits"]["total"] == 1
    assert hits[0]["id"] == comm.data["id"]
    assert "links" in hits[0]
    # featured entries should not show up in search results
    assert "featured" not in hits[0]

    # new community with only future entry should not show up in search
    c2 = community_service.create(
        identity=system_identity, data={**comm.data, "slug": "c2-id"}
    )
    community_service.featured_create(
        identity=system_identity, community_id=c2.data["id"], data=future_data
    ).to_dict()
    featured_comms = community_service.featured_search(
        identity=system_identity
    ).to_dict()
    hits = featured_comms["hits"]["hits"]
    assert len(hits) == featured_comms["hits"]["total"] == 1
    assert hits[0]["id"] == comm.data["id"]

    # adding past featured entry to new community. first community should show up first
    data["start_date"] = (datetime.utcnow() - timedelta(days=1)).isoformat()
    community_service.featured_create(
        identity=system_identity, community_id=c2.data["id"], data=data
    ).to_dict()
    featured_comms = community_service.featured_search(
        identity=system_identity
    ).to_dict()
    hits = featured_comms["hits"]["hits"]
    assert len(hits) == featured_comms["hits"]["total"] == 2
    assert hits[0]["id"] == comm.data["id"]
    assert hits[1]["id"] == c2.data["id"]

    # adding more current past featured entry to new community. new community should show up first
    data["start_date"] = datetime.utcnow().isoformat()
    community_service.featured_create(
        identity=system_identity, community_id=c2.data["id"], data=data
    ).to_dict()
    featured_comms = community_service.featured_search(
        identity=system_identity
    ).to_dict()
    hits = featured_comms["hits"]["hits"]
    assert len(hits) == featured_comms["hits"]["total"] == 2
    assert hits[0]["id"] == c2.data["id"]
    assert hits[1]["id"] == comm.data["id"]


def test_reindex_featured_entries_task(community_service, comm, db, search_clear):
    """Test that reindexing task works."""
    tomorrow = {
        "start_date": (datetime.utcnow() + timedelta(days=1)).isoformat(),
    }
    c2 = community_service.create(
        identity=system_identity, data={**comm.data, "slug": "c2-id"}
    )
    community_service.featured_create(
        identity=system_identity, community_id=c2.data["id"], data=tomorrow
    ).to_dict()

    near_future_difference = 2
    near_future = {
        "start_date": (
            datetime.utcnow() + timedelta(seconds=near_future_difference)
        ).isoformat(),
    }
    community_service.featured_create(
        identity=system_identity, community_id=comm.data["id"], data=near_future
    ).to_dict()
    featured_comms = community_service.featured_search(
        identity=system_identity
    ).to_dict()
    hits = featured_comms["hits"]["hits"]
    assert len(hits) == featured_comms["hits"]["total"] == 0

    time.sleep(near_future_difference)
    reindex_featured_entries()

    # only one community should be reindexed and returned
    featured_comms = community_service.featured_search(
        identity=system_identity
    ).to_dict()
    hits = featured_comms["hits"]["hits"]
    assert len(hits) == featured_comms["hits"]["total"] == 1
    assert hits[0]["id"] == comm.data["id"]


def test_create_featured(community_service, comm, comm_restricted):
    """Test that a featured entry can be created."""
    data = {
        "start_date": datetime.utcnow().isoformat(),
    }

    f = community_service.featured_create(
        identity=system_identity, community_id=comm.data["id"], data=data
    ).to_dict()
    assert f["start_date"] == data["start_date"]
    assert f["id"]

    # can not create entry for a non-public community
    with pytest.raises(ValidationError):
        community_service.featured_create(
            identity=system_identity, community_id=comm_restricted.data["id"], data=data
        )

    # can not create entry with invalid data
    with pytest.raises(ValidationError):
        community_service.featured_create(
            identity=system_identity, community_id=comm.data["id"], data={}
        )


def test_get_featured(community_service, comm):
    """Test that featured entries are indexed and returned correctly."""
    data = {
        "start_date": datetime.utcnow().isoformat(),
    }
    future_data = {
        "start_date": (datetime.utcnow() + timedelta(days=1)).isoformat(),
    }

    community_service.featured_create(
        identity=system_identity, community_id=comm.data["id"], data=data
    )
    community_service.featured_create(
        identity=system_identity, community_id=comm.data["id"], data=future_data
    )
    featured = community_service.featured_list(
        identity=system_identity, community_id=comm.data["id"]
    ).to_dict()
    assert len(featured["hits"]["hits"]) == featured["hits"]["total"] == 2
    assert featured["hits"]["hits"][0]["start_date"] == data["start_date"]
    assert featured["hits"]["hits"][1]["start_date"] == future_data["start_date"]


def test_delete_featured(community_service, comm):
    """Test that featured entries are deleted correctly."""
    data = {
        "start_date": datetime.utcnow().isoformat(),
    }
    future_data = {
        "start_date": (datetime.utcnow() + timedelta(days=1)).isoformat(),
    }

    past_entry = community_service.featured_create(
        identity=system_identity, community_id=comm.data["id"], data=data
    ).to_dict()
    community_service.featured_create(
        identity=system_identity, community_id=comm.data["id"], data=future_data
    ).to_dict()
    featured = community_service.featured_list(
        identity=system_identity, community_id=comm.data["id"]
    ).to_dict()

    assert len(featured["hits"]["hits"]) == featured["hits"]["total"] == 2

    community_service.featured_delete(
        identity=system_identity,
        community_id=comm.data["id"],
        featured_id=past_entry["id"],
    )
    featured = community_service.featured_list(
        identity=system_identity, community_id=comm.data["id"]
    ).to_dict()
    assert len(featured["hits"]["hits"]) == featured["hits"]["total"] == 1
    assert featured["hits"]["hits"][0]["start_date"] == future_data["start_date"]

    # Error when trying to delete entry which is already deleted
    with pytest.raises(CommunityFeaturedEntryDoesNotExistError):
        community_service.featured_delete(
            identity=system_identity,
            community_id=comm.data["id"],
            featured_id=past_entry["id"],
        )

    # Error when trying to delete entry which does not exist
    with pytest.raises(CommunityFeaturedEntryDoesNotExistError):
        community_service.featured_delete(
            identity=system_identity, community_id=comm.data["id"], featured_id=9999
        )


def test_update_featured(community_service, comm):
    """Test that featured entries are indexed and returned correctly."""
    data = {
        "start_date": datetime.utcnow().isoformat(),
    }
    future_data = {
        "start_date": (datetime.utcnow() + timedelta(days=1)).isoformat(),
    }

    past_entry = community_service.featured_create(
        identity=system_identity, community_id=comm.data["id"], data=data
    ).to_dict()
    updated_entry = community_service.featured_update(
        identity=system_identity,
        community_id=comm.data["id"],
        featured_id=past_entry["id"],
        data=future_data,
    ).to_dict()
    assert updated_entry["id"] == past_entry["id"]
    assert updated_entry["start_date"] != past_entry["start_date"]

    featured = community_service.featured_list(
        identity=system_identity, community_id=comm.data["id"]
    ).to_dict()
    assert len(featured["hits"]["hits"]) == featured["hits"]["total"] == 1
    assert featured["hits"]["hits"][0]["start_date"] == future_data["start_date"]

    # Error when trying to update entry which does not exist
    with pytest.raises(CommunityFeaturedEntryDoesNotExistError):
        community_service.featured_update(
            identity=system_identity,
            community_id=comm.data["id"],
            featured_id=9999,
            data=future_data,
        ).to_dict()

    # Error when trying to update entry with invalid data
    with pytest.raises(ValidationError):
        x = community_service.featured_update(
            identity=system_identity,
            community_id=comm.data["id"],
            featured_id=past_entry["id"],
            data={},
        ).to_dict()

    community_service.featured_delete(
        identity=system_identity,
        community_id=comm.data["id"],
        featured_id=past_entry["id"],
    )

    # Error when trying to update entry which is already deleted
    with pytest.raises(CommunityFeaturedEntryDoesNotExistError):
        community_service.featured_update(
            identity=system_identity,
            community_id=comm.data["id"],
            featured_id=past_entry["id"],
            data=future_data,
        ).to_dict()


def test_cleanup_pre_search(db, search_clear):
    """Cleanup database and search for following tests.

    Tests before do not depend on a clean db or es. Cleanup is also an
    expensive task so te tests will run faster without them.
    """
    pass


def test_search_user(
    app,
    db,
    search_clear,
    location,
    anon_identity,
    community_service,
    community,
    members,
    new_user,
):
    current_cache.clear()
    owner = members["owner"]
    reader = members["reader"]

    # Community members see them
    hits = community_service.search_user_communities(identity=owner.identity).to_dict()[
        "hits"
    ]["hits"]

    assert 1 == len(hits)
    assert {h["id"] for h in hits} & {community["id"]}

    hits = community_service.search_user_communities(
        identity=reader.identity
    ).to_dict()["hits"]["hits"]
    assert 1 == len(hits)
    assert {h["id"] for h in hits} & {community["id"]}

    # Non-community members don't see them
    hits = community_service.search_user_communities(
        identity=new_user.identity
    ).to_dict()["hits"]["hits"]
    assert 0 == len(hits)

    with pytest.raises(PermissionDeniedError):
        community_service.search_user_communities(identity=anon_identity)


def test_search_community_requests(
    app,
    db,
    search_clear,
    location,
    anon_identity,
    community_service,
    community,
    members,
    new_user,
):
    """Test who cannot see community requests.

    Community managers should be the one able to see the community requests
    This test should happen in invenio-rdm-records where the review service is defined.
    """
    reader = members["reader"]

    # Community members don't see them
    with pytest.raises(PermissionDeniedError):
        community_service.search_community_requests(
            identity=reader.identity, community_id=community.id
        )

    # Non-community members don't see them
    with pytest.raises(PermissionDeniedError):
        community_service.search_community_requests(
            identity=new_user.identity, community_id=community.id
        )

    # Anonymous users don't see them
    with pytest.raises(PermissionDeniedError):
        community_service.search_community_requests(
            identity=anon_identity, community_id=community.id
        )


#
# Deletion workflows
#


def test_community_deletion(community_service, users, comm):
    """Test simple community deletion of a community."""
    user = users["owner"].user
    community = comm

    assert community._obj.deletion_status == CommunityDeletionStatusEnum.PUBLISHED

    # delete the community
    tombstone_info = {"note": "no specific reason, tbh"}
    community = community_service.delete(system_identity, community.id, tombstone_info)
    tombstone = community._obj.tombstone

    # check if the tombstone information got added as expected
    assert community._obj.deletion_status == CommunityDeletionStatusEnum.DELETED
    assert tombstone.is_visible
    assert tombstone.removed_by == {"user": "system"}
    assert tombstone.removal_reason is None
    assert tombstone.note == tombstone_info["note"]
    assert isinstance(tombstone.citation_text, str)
    assert arrow.get(tombstone.removal_date).date() == datetime.utcnow().date()

    # mark the community for purge
    community = community_service.mark_community_for_purge(
        system_identity, community.id
    )
    assert community._obj.deletion_status == CommunityDeletionStatusEnum.MARKED
    assert community._obj.deletion_status.is_deleted
    assert community._obj.tombstone is not None

    # remove the mark again, we don't wanna purge it after all
    community = community_service.unmark_community_for_purge(
        system_identity, community.id
    )
    assert community._obj.deletion_status == CommunityDeletionStatusEnum.DELETED
    assert community._obj.deletion_status.is_deleted
    assert community._obj.tombstone is not None

    # restore the community, it wasn't so bad after all
    community = community_service.restore_community(system_identity, community.id)
    assert community._obj.deletion_status == CommunityDeletionStatusEnum.PUBLISHED
    assert not community._obj.deletion_status.is_deleted
    assert community._obj.tombstone is None


def test_invalid_community_deletion_workflows(community_service, comm):
    """Test the wrong order of deletion operations."""
    assert comm._obj.deletion_status == CommunityDeletionStatusEnum.PUBLISHED

    # we cannot restore a published community
    with pytest.raises(DeletionStatusError):
        community_service.restore_community(system_identity, comm.id)

    # we cannot mark a published community for purge
    with pytest.raises(DeletionStatusError):
        community_service.mark_community_for_purge(system_identity, comm.id)

    # we cannot unmark a published community
    with pytest.raises(DeletionStatusError):
        community_service.unmark_community_for_purge(system_identity, comm.id)

    comm = community_service.delete(system_identity, comm.id, {})
    assert comm._obj.deletion_status == CommunityDeletionStatusEnum.DELETED

    # we cannot unmark a deleted community
    with pytest.raises(DeletionStatusError):
        community_service.unmark_community_for_purge(system_identity, comm.id)

    comm = community_service.mark_community_for_purge(system_identity, comm.id)
    assert comm._obj.deletion_status == CommunityDeletionStatusEnum.MARKED

    # we cannot directly restore a community marked for purge
    with pytest.raises(DeletionStatusError):
        community_service.restore_community(system_identity, comm.id)


def test_get_cached_community_slug(community_service, comm, db, search_clear):
    """Test get_cached_community_slug."""
    c1 = community_service.create(
        identity=system_identity, data={**comm.data, "slug": "myslug"}
    )
    c2 = community_service.create(
        identity=system_identity, data={**comm.data, "slug": "myslug2"}
    )
    # reset
    get_cached_community_slug.cache_clear()
    # cache miss
    slug = get_cached_community_slug(c1.id, community_service.id)
    assert slug == "myslug"
    hits, misses = get_cached_community_slug.cache_info()
    assert (hits, misses) == (0, 1)
    # cache hit
    slug = get_cached_community_slug(c1.id, community_service.id)
    assert slug == "myslug"
    hits, misses = get_cached_community_slug.cache_info()
    assert (hits, misses) == (1, 1)
    # cache miss
    slug = get_cached_community_slug(c2.id, community_service.id)
    assert slug == "myslug2"
    hits, misses = get_cached_community_slug.cache_info()
    assert (hits, misses) == (1, 2)

    with pytest.raises(PIDDoesNotExistError):
        random_uuid = uuid.uuid4()
        get_cached_community_slug(random_uuid, community_service.id)


def test_theme_updates(
    app,
    db,
    search_clear,
    location,
    anon_identity,
    community_service,
    community,
    members,
    new_user,
):
    current_cache.clear()
    owner = members["owner"]

    theme_data = {
        "theme": {
            "style": {
                "primaryColor": "#004494",
                "tertiaryColor": "#e3eefd",
                "secondaryColor": "#FFD617",
                "primaryTextColor": "#FFFFFF",
                "tertiaryTextColor": "#1c5694",
                "secondaryTextColor": "#000000",
                "mainHeaderBackgroundColor": "#FFFFFF",
                "font": {"size": "16px", "family": "Arial, sans-serif", "weight": 600},
            },
            "brand": "horizon",
        }
    }

    # Update theme
    community_data = deepcopy(community.data)
    community_data.update(theme_data)

    # check if owner can update theme (early stage: it should not be possible)
    with pytest.raises(PermissionDeniedError):
        community_service.update(owner.identity, community.id, community_data)

    # only system can update the theme
    community_service.update(system_identity, community.id, community_data)
    community_item = community_service.read(system_identity, community.id)
    assert community_item.data["theme"]["brand"] == "horizon"

    # Update community data without passing theme should keep the stored theme
    community_data = deepcopy(community_item.data)
    community_data.pop("theme")

    community_service.update(system_identity, community.id, community_data)
    # Refresh index
    community_service.record_cls.index.refresh()

    # owner, anon, and system should be able to read the theme and see in search
    for idty in (owner.identity, anon_identity, system_identity):
        community_item = community_service.read(idty, community.id)
        assert community_item.data["theme"]["brand"] == "horizon"
        community_search = community_service.search(idty)
        assert next(community_search.hits)["theme"]["brand"] == "horizon"

    # Delete theme by setting to None
    community_data = deepcopy(community_item.data)
    community_data["theme"] = None

    # check if owner can delete theme (early stage: it should not be possible)
    with pytest.raises(PermissionDeniedError):
        community_service.update(owner.identity, community.id, community_data)

    # only system can delete the theme
    community_service.update(system_identity, community.id, community_data)
    community_item = community_service.read(system_identity, community.id)
    assert community_item.data.get("theme") is None

    # Set {theme: None} to a community that doesn't have any stored theme should not
    # store None value
    community_data = deepcopy(community_item.data)
    community_data["theme"] = None
    community_service.update(system_identity, community.id, community_data)
    community_item = community_service.read(system_identity, community.id)
    assert community_item.data.get("theme") is None
    assert "theme" not in community_item.data


def test_children_updates(
    app,
    db,
    search_clear,
    location,
    community_service,
    community,
    members,
):
    current_cache.clear()
    owner = members["owner"]

    # Update children
    community_data = deepcopy(community.data)
    community_data.update(dict(children=dict(allow=True)))

    # check if owner can update the children
    with pytest.raises(PermissionDeniedError):
        community_service.update(owner.identity, community.id, community_data)

    # only system can update the children
    community_service.update(system_identity, community.id, community_data)
    community_item = community_service.read(system_identity, community.id)
    assert community_item.data["children"]["allow"] == True

    # Update community data without passing children should keep the stored values
    community_data = deepcopy(community_item.data)
    community_data.pop("children")

    # Owner should be able to perfrom this operation since is not changing the children
    community_service.update(owner.identity, community.id, community_data)
    # Refresh index
    community_service.record_cls.index.refresh()


def test_parent_create(community_service, comm):
    parent = comm
    community_service.update(
        identity=system_identity,
        id_=str(parent.id),
        data={**parent.data, "children": {"allow": True}},
    )
    child = community_service.create(
        identity=system_identity,
        data={**comm.data, "slug": "child", "parent": {"id": str(parent.id)}},
    )
    assert str(child._obj.parent.id) == parent.id


def test_parent_update(community_service, comm):
    parent = comm
    community_service.update(
        identity=system_identity,
        id_=str(parent.id),
        data={**parent.data, "children": {"allow": True}},
    )
    child = community_service.create(
        identity=system_identity, data={**comm.data, "slug": "child1"}
    )

    child = community_service.update(
        identity=system_identity,
        id_=str(child.id),
        data={**child.data, "parent": {"id": str(parent.id)}},
    )
    assert str(child._obj.parent.id) == parent.id


def test_parent_remove(community_service, comm):
    parent = comm
    community_service.update(
        identity=system_identity,
        id_=str(parent.id),
        data={**parent.data, "children": {"allow": True}},
    )
    child = community_service.create(
        identity=system_identity,
        data={**comm.data, "slug": "child2", "parent": {"id": str(parent.id)}},
    )

    child = community_service.update(
        identity=system_identity, id_=str(child.id), data={**child.data, "parent": None}
    )
    assert child._obj.parent == None


def test_update_parent_community_not_exists(community_service, comm):
    child = comm

    with pytest.raises(ValidationError) as e:
        community_service.update(
            identity=system_identity,
            id_=str(child.id),
            data={**child.data, "parent": {"id": str(uuid.uuid4())}},
        )


def test_parent_update_parent_children_not_allowed(community_service, comm):
    parent = comm

    child = community_service.create(
        identity=system_identity, data={**comm.data, "slug": "child4"}
    )

    with pytest.raises(ValidationError) as e:
        community_service.update(
            identity=system_identity,
            id_=str(child.id),
            data={**child.data, "parent": {"id": str(parent.id)}},
        )


def test_parent_update_child_children_are_allowed(community_service, comm):
    parent = comm

    child = community_service.create(
        identity=system_identity, data={**comm.data, "slug": "child5"}
    )
    community_service.update(
        identity=system_identity,
        id_=str(parent.id),
        data={**parent.data, "children": {"allow": True}},
    )
    child = community_service.update(
        identity=system_identity,
        id_=str(child.id),
        data={**child.data, "children": {"allow": True}},
    )

    with pytest.raises(ValidationError) as e:
        community_service.update(
            identity=system_identity,
            id_=str(child.id),
            data={**child.data, "parent": {"id": str(parent.id)}},
        )


def test_bulk_update_parent(
    community_service, parent_community, comm, restricted_community
):
    """Test bulk add parent to children."""
    children = [comm.id, restricted_community.id]
    parent_community.children.allow = True
    parent_community.commit()
    community_service.bulk_update_parent(system_identity, children, parent_community.id)
    for c_id in children:
        c_comm = community_service.record_cls.pid.resolve(c_id)
        assert str(c_comm.parent.id) == str(parent_community.id)


def test_bulk_update_parent_overwrite(
    community_service, parent_community, comm, restricted_community
):
    """Test bulk update parent of communities that are already parented."""
    children = [comm.id]
    parent_community.children.allow = True
    parent_community.commit()
    community_service.bulk_update_parent(system_identity, children, parent_community.id)
    for c_id in children:
        c_comm = community_service.record_cls.pid.resolve(c_id)
        assert str(c_comm.parent.id) == str(parent_community.id)

    children = [comm.id, restricted_community.id]
    community_service.bulk_update_parent(system_identity, children, parent_community.id)
    for c_id in children:
        c_comm = community_service.record_cls.pid.resolve(c_id)
        assert str(c_comm.parent.id) == str(parent_community.id)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Northwestern University.
# Copyright (C) 2022 CERN.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invitation request type."""

from invenio_access.permissions import system_identity
from invenio_i18n import lazy_gettext as _
from invenio_notifications.services.uow import NotificationOp
from invenio_requests.customizations import RequestType, actions

from invenio_communities.notifications.builders import (
    CommunityInvitationAcceptNotificationBuilder,
    CommunityInvitationCancelNotificationBuilder,
    CommunityInvitationDeclineNotificationBuilder,
    CommunityInvitationExpireNotificationBuilder,
)

from ...proxies import current_communities


def service():
    """Service."""
    return current_communities.service.members


#
# Actions
#
# All actions use `system_identity` and not the `identity` param, because
# the permission check happens at the request service (`execute_action`) level
# before reaching these components and therefore the check is not needed.
# These actions will assert the identity is `system identity`, which cannot
# be obtained as a user.
class AcceptAction(actions.AcceptAction):
    """Accept action."""

    def execute(self, identity, uow):
        """Execute action."""
        service().accept_invite(system_identity, self.request.id, uow=uow)
        uow.register(
            NotificationOp(
                CommunityInvitationAcceptNotificationBuilder.build(self.request)
            )
        )
        super().execute(identity, uow)


class DeclineAction(actions.DeclineAction):
    """Delete action."""

    def execute(self, identity, uow):
        """Execute action."""
        service().decline_invite(system_identity, self.request.id, uow=uow)
        uow.register(
            NotificationOp(
                CommunityInvitationDeclineNotificationBuilder.build(self.request)
            )
        )
        super().execute(identity, uow)


class CancelAction(actions.CancelAction):
    """Cancel action."""

    def execute(self, identity, uow):
        """Execute action."""
        service().decline_invite(system_identity, self.request.id, uow=uow)
        uow.register(
            NotificationOp(
                CommunityInvitationCancelNotificationBuilder.build(self.request)
            )
        )
        super().execute(identity, uow)


class ExpireAction(actions.ExpireAction):
    """Expire action."""

    def execute(self, identity, uow):
        """Execute action."""
        service().decline_invite(system_identity, self.request.id, uow=uow)
        uow.register(
            NotificationOp(
                CommunityInvitationExpireNotificationBuilder.build(self.request)
            )
        )
        super().execute(identity, uow)


#
# Request
#
class CommunityInvitation(RequestType):
    """Community member invitation request type."""

    type_id = "community-invitation"
    name = _("Community invitation")

    create_action = "create"
    available_actions = {
        "create": actions.CreateAndSubmitAction,
        "delete": actions.DeleteAction,
        "accept": AcceptAction,
        "decline": DeclineAction,
        "cancel": CancelAction,
        "expire": ExpireAction,
    }

    creator_can_be_none = False
    topic_can_be_none = False
    allowed_creator_ref_types = ["community"]
    allowed_receiver_ref_types = ["user"]
    allowed_topic_ref_types = ["community"]

    needs_context = {
        "community_roles": [
            "owner",
            "manager",
        ]
    }
